###################################################
### chunk number 1: preliminaries
###################################################
library("ReacTran")
options(prompt = "> ")
options(width=75)


###################################################
### chunk number 2: 
###################################################
 (grid<-setup.grid.1D(L=100,dx.1=1,N=50))


###################################################
### chunk number 3: Grid
###################################################
 plot(grid)


###################################################
### chunk number 4: grid
###################################################
 plot(grid)


###################################################
### chunk number 5: 
###################################################

 grid <- setup.grid.1D(L=10,N=100)
 Db   <- setup.prop.1D(func=p.exp,grid=grid,y.0=5,y.inf=0,x.L= 2)


###################################################
### chunk number 6: 
###################################################
 exp.inc <- function(x,y.0=1,y.inf=0.5,x.L=0,x.att=1)
       return(1-p.exp(x,y.0,y.inf,x.L,x.att))

 VFsolid <- setup.prop.1D(func=exp.inc,grid=grid,y.0=0.9,y.inf=0.7)



###################################################
### chunk number 7: prop
###################################################
  par(mfrow=c(1,2))
  plot(VFsolid,grid,xyswap=TRUE,type="l",main="1-porosity")
  plot(Db,grid,xyswap=TRUE,type="l",main="Db")
  par(mfrow=c(1,1))


###################################################
### chunk number 8: prop
###################################################
  par(mfrow=c(1,2))
  plot(VFsolid,grid,xyswap=TRUE,type="l",main="1-porosity")
  plot(Db,grid,xyswap=TRUE,type="l",main="Db")
  par(mfrow=c(1,1))


###################################################
### chunk number 9: 
###################################################
tran.1D(C=1:20, D=0, flux.up = 1, v=1, dx=1)
tran.1D(C=1:20, D=0, flux.up = 1, v=1, dx=1, full.output=TRUE)


###################################################
### chunk number 10: 
###################################################
    parms <- c(F0 = 1, v = 1, k = 0.1, dx = 1)


###################################################
### chunk number 11: 
###################################################
    advModel <- function(t, C, parms) {
    
      with (as.list(parms), {

        Tran <- tran.1D(C=C, D=0, flux.up = F0, v=v, dx=dx)
        Consumption <-  k*C
        dC   <- Tran$dC - Consumption
        
        return (list(dC = dC, Consumption= Consumption,
                     flux.up = Tran$flux.up, flux.down = Tran$flux.down))
      })

    }



###################################################
### chunk number 12: 
###################################################
  out <- steady.1D(func=advModel, y=runif(25), parms=parms,
                     nspec=1, positive=TRUE)



###################################################
### chunk number 13: 
###################################################
out


###################################################
### chunk number 14: st1
###################################################
plot (out$y ,type="l",xlab="x", ylab="Conc")


###################################################
### chunk number 15: figst1
###################################################
plot (out$y ,type="l",xlab="x", ylab="Conc")


###################################################
### chunk number 16: 
###################################################
with (out, print(sum(Consumption)-(flux.up-flux.down)))


###################################################
### chunk number 17: 
###################################################

Aggregate.Model <- function(time,O2,pars) {

  tran <- tran.1D(C=O2, C.down=C.ow.O2,
    D=D.grid, A=A.grid,
    VF=por.grid, dx=grid )

  reac <- - R.O2*(O2/(Ks+O2))
  return(list(dCdt = tran$dC + reac, reac = reac,
              flux.up = tran$flux.up, flux.down = tran$flux.down))

}


###################################################
### chunk number 18: 
###################################################
C.ow.O2 <- 0.25     # concentration O2 water [micromol cm-3]
por     <- 0.8      # porosity
D       <- 400      # diffusion coefficient O2 [cm2 yr-1]
v       <- 0        # advective velocity [cm yr-1]
R.O2    <- 1000000  # O2 consumption rate [micromol cm-3 yr-1]
Ks      <- 0.005    # O2 saturation constant [micromol cm-3]


###################################################
### chunk number 19: 
###################################################
R <- 0.025           # radius of the agggregate [cm]
N <- 100             # number of grid layers
grid <- setup.grid.1D(x.up=0,L=R,N=N)


###################################################
### chunk number 20: 
###################################################
por.grid <- setup.prop.1D(value=por,grid=grid)
D.grid <- setup.prop.1D(value=D,grid=grid)


###################################################
### chunk number 21: 
###################################################
sphere.surf <- function (x) 4*pi*x^2
A.grid  <- setup.prop.1D(func=sphere.surf,grid=grid)


###################################################
### chunk number 22: 
###################################################
O2.agg <- steady.1D (y = runif(N), func=Aggregate.Model, nspec=1,
                     positive=TRUE, atol=1e-10)


###################################################
### chunk number 23: agg
###################################################
par(mfrow=c(1,1))

plot(grid$x.mid,O2.agg$y,xlab="distance from centre, cm",
ylab="mmol/m3",
main="Diffusion-reaction of O2 in a spherical aggregate")
legend ("bottomright",pch=c(1,18),lty=1,col=c("black"),
        c("O2 concentration"))


###################################################
### chunk number 24: figagg
###################################################
par(mfrow=c(1,1))

plot(grid$x.mid,O2.agg$y,xlab="distance from centre, cm",
ylab="mmol/m3",
main="Diffusion-reaction of O2 in a spherical aggregate")
legend ("bottomright",pch=c(1,18),lty=1,col=c("black"),
        c("O2 concentration"))


###################################################
### chunk number 25: 
###################################################
  O2.agg$flux.up
  O2.agg$flux.down


###################################################
### chunk number 26: 
###################################################
 Volume <- A.grid$mid * grid$dx
(Consump <- - sum(O2.agg$reac * Volume * por.grid$mid))


###################################################
### chunk number 27: 
###################################################
 (Fluxin <- - O2.agg$flux.down*A.grid$int[N+1])


###################################################
### chunk number 28: 
###################################################
  Consump - Fluxin


###################################################
### chunk number 29: 
###################################################
river.model <- function (t=0,OC,pars=NULL)
{
  tran <- tran.volume.1D(C=OC,F.up=F.OC,F.lat=F.lat,Disp=Disp,
      flow=flow,V=Volume)
  reac <- - k*OC

  return(list(dCdt = tran$dC + reac,
            F.up = tran$F.up, F.down = tran$F.down,
            F.lat = sum(tran$F.lat)))
}


###################################################
### chunk number 30: 
###################################################
nbox          <- 500                # number of grid cells
lengthEstuary <- 100000             # length of estuary [m]
BoxLength     <- lengthEstuary/nbox # [m]


###################################################
### chunk number 31: 
###################################################
Distance      <- seq(BoxLength/2, by=BoxLength, len=nbox) # [m]

CrossArea <- 4000 + 72000 * Distance^5 /(Distance^5+50000^5)

Volume  <- CrossArea*BoxLength


###################################################
### chunk number 32: 
###################################################
Disp    <- 1000   # m3/s, bulk dispersion coefficient
flow    <- 180    # m3/s, mean river flow


###################################################
### chunk number 33: 
###################################################
F.OC    <- 180               # input organic carbon [mol s-1]
F.lat.0 <- F.OC              # lateral input organic carbon [mol s-1]

k       <- 10/(365*24*3600)  # decay constant organic carbon [s-1]


###################################################
### chunk number 34: 
###################################################
F.lat <- rep(0,length.out=nbox)


###################################################
### chunk number 35: 
###################################################
sol  <- steady.1D(y=runif(nbox), fun=river.model, nspec=1, atol=1e-15,
                    rtol=1e-15, positive = TRUE)
Conc1 <- sol$y


###################################################
### chunk number 36: 
###################################################
F.lat <- F.lat.0*dnorm(x=Distance/lengthEstuary,
                       mean = Distance[nbox/2]/lengthEstuary,
                       sd = 1/20, log = FALSE) /nbox
sol2<- steady.1D(runif(nbox), fun=river.model, nspec=1, atol=1e-15,
                    rtol=1e-15, positive = TRUE)
Conc2 <- sol2$y


###################################################
### chunk number 37: est
###################################################
matplot(Distance/1000,cbind(Conc1,Conc2),lwd=2,
main="Organic carbon decay in an estuary",xlab="distance [km]",
ylab="OC Concentration [mM]",
type="l")
legend ("topright",lty=1,col=c("black","red"),
        c("baseline","with lateral input"))


###################################################
### chunk number 38: est
###################################################
matplot(Distance/1000,cbind(Conc1,Conc2),lwd=2,
main="Organic carbon decay in an estuary",xlab="distance [km]",
ylab="OC Concentration [mM]",
type="l")
legend ("topright",lty=1,col=c("black","red"),
        c("baseline","with lateral input"))


###################################################
### chunk number 39: 
###################################################
sol$F.up + sol$F.lat - sol$F.down - sum(sol$y*k*Volume)


###################################################
### chunk number 40: 
###################################################
n     <- 100           # number of grid cells
dy    <- dx <- 100/n   # grid size

Dy    <- Dx <- 5   # diffusion coeff, X- and Y-direction
r     <- -0.02     # production/consumption rate
Bc    <- 300       # boundary concentration
irr   <- 20        # irrigation rate
vx    <- 1         # advection


###################################################
### chunk number 41: 
###################################################
y  <- matrix(nr=n,nc=n,0)


###################################################
### chunk number 42: 
###################################################
Diff2D <- function (t, y, parms, N)

{
  CONC <- matrix(nr=N, nc=N, y)
# Transport

   Tran    <-tran.2D(CONC, D.x=Dx, D.y=Dy, C.y.down=Bc,
                     dx=dx, dy=dy, v.x = vx)

# transport + reaction
   dCONC   <- Tran$dC + r*CONC

# Bioirrigation in a central spot
    mid <- N/2
    dCONC[mid,mid] <- dCONC[mid,mid]  + irr*(Bc-CONC[mid,mid])

  return (list(as.vector(dCONC)))
}


###################################################
### chunk number 43: 
###################################################
print(system.time(
std  <-  steady.2D(func=Diff2D, y=as.vector(y), time=0, N=n,
          parms=NULL, lrw=1000000, dimens=c(n,n),
          nout=0, positive=TRUE)
))


###################################################
### chunk number 44: 
###################################################
times <- seq(0,100,5)
print(system.time(
  out2  <-  ode.2D(func=Diff2D, y=as.vector(y), times=times, N=n,
            parms=NULL, lrw=10000000, dimens=c(n,n))
))


###################################################
### chunk number 45: twod5
###################################################
mat  <-  matrix(nr=n,nc=n,out2[5,-1])
filled.contour(mat,zlim=c(0,Bc), color=femmecol,main="after 20 time units")


###################################################
### chunk number 46: twod
###################################################
mat  <-  matrix(nr=n,nc=n,out2[5,-1])
filled.contour(mat,zlim=c(0,Bc), color=femmecol,main="after 20 time units")


###################################################
### chunk number 47: twodst
###################################################
mat  <-  matrix(nr=n,nc=n,std$y)
filled.contour(mat,zlim=c(0,Bc), color=femmecol,main="steady-state")


###################################################
### chunk number 48: twodst
###################################################
mat  <-  matrix(nr=n,nc=n,std$y)
filled.contour(mat,zlim=c(0,Bc), color=femmecol,main="steady-state")


